#!/usr/bin/ruby
# encoding: utf-8

class AntibotAviraScanner
	
	def initialize(drvscreen, params, logdir=nil)
		@drivescreen = drvscreen 
		@params = params
		@logdir = logdir
		@qparams = params.collect { |x| "'#{x}'" }  
		@infections = Hash.new
		@return_codes = Hash.new
		@killed = false
		@partcount = -1
		@logdb = nil
		@tstamp = Time.now.to_i
		@debug = AntibotMisc.boot_options["debug"]
		@avira_version = nil
		prepare_logdb
	end
	attr_reader :infections, :killed, :return_codes, :tstamp, :params
	attr_accessor :logbd
	
	def kill_scan
		system("killall -9 scancl")
		@killed = true
	end
	
	def prepare_logdb
		return true unless @logdb.nil?
		system("mkdir -p /var/log/antibot3")
		@logdb = SQLite3::Database.new(@logdir + "/fulllog.sqlite") unless @logdir.nil?
		@logdb = SQLite3::Database.new("/var/log/antibot3/fulllog.sqlite") if @logdir.nil?
		begin
			@logdb.execute("CREATE TABLE single_infection " + 
				" (id INTEGER PRIMARY KEY ASC, scantime INTEGER(12), uuid VARCHAR(80), fpath VARCHAR(200), bakup VARCHAR(200), "  + 
				"  infection VARCHAR(120), scancl VARCHAR(120), sha1pre CHAR(40), sha1post CHAR(40), retvalue INT(6), " + 
				" fileout VARCHAR(120), scanout VARCHAR(800), action CHAR(20), retval INTEGER(5) ); ")
		rescue
		end
		# FIXME! Create indices!
	end
	
	def prepare_scan(pgbar=nil, label=nil)
		puts "Preparing scan:"
		now = Time.new.strftime("%Y%m%d-%H%M%S") 
		system("blkid | grep -v squashfs > '#{@logdir}/#{now}-laufwerke.txt' ") unless @logdir.nil?
		@partcount = 0
		@drivescreen.dump_marked
		( @drivescreen.int_drives + @drivescreen.ext_drives ).each { |d| 
			d.partitions.each { |p|
				if @drivescreen.saved_states[p.uuid] == true
					puts "Marked for scan: " + p.uuid
					p.filecount(pgbar)
					@partcount += 1
				end
			}
		}
		unless pgbar.nil?
			pgbar.text = @partcount.to_s + " Partitionen zu scannen"
			pgbar.text = "Virenscan abgebrochen" if @killed == true
		end
	end
	
	def check_avira(pgbar=nil)
		return true, 0 unless @avira_version.nil?
		@avira_version = Array.new
		scancl = "ruby generic_wrapper.rb /var/run/scancl.version.ret /AntiVir/scancl --version"
		lastupd = Time.now.to_f
		IO.popen(scancl) { |l|
			while l.gets
				line = $_ 
				unless line.strip == "-TickTock-" 
					@avira_version.push(line)
					puts line
				end
				unless pgbar.nil? || Time.now.to_f - lastupd < 0.1
					lastupd = Time.now.to_f
					pgbar.pulse
					pgbar.text = "Prüfe Funktionalität von Avira"
					while (Gtk.events_pending?)
						Gtk.main_iteration
					end
				end
			end
		}
		retval = File.read("/var/run/scancl.version.ret").strip.to_i
		@avira_version = nil if retval > 1
		return false, retval if @avira_version.nil?
	end
	
	# interval: delta in seconds (integer) for the signature update
	# FIXME! Use proxy settings
	# FIXME! Change avupdate script to return proper exit code!
	
	def update_signatures(interval, parameters=[], pgbar=nil)
		return false, "Schlüsseldatei fehlt!" unless File.exist?("/AntiVir/rescue_cd.key")
		return true, nil if File.stat("/AntiVir/rescue_cd.key").mtime.to_f + interval.to_f > Time.now.to_f 
		lastupd = Time.now.to_f
		now = Time.new.strftime("%Y%m%d-%H%M%S") 
		log = File.new(@logdir + "/" + now + "-avupdate.txt", "w") unless @logdir.nil?
		log = File.new(@logdir + "/" + now + "-avupdate.txt", "w") if @logdir.nil? && @debug == true
		command = "ruby generic_wrapper.rb /var/run/avupdate.ret /AntiVirUpdate/avupdate"
		parameters.each { |p| command = command + " '#{p}'" } 
		IO.popen(command) { |l|
			while l.gets
				line = $_
				unless pgbar.nil? || Time.now.to_f - lastupd < 0.1
					lastupd = Time.now.to_f
					pgbar.pulse
					pgbar.text = "Aktualisiere Virensignaturen"
					while (Gtk.events_pending?)
						Gtk.main_iteration
					end
				end
				unless line.strip == "-TickTock-" 
					puts line
					log.write(line) unless @logdir.nil?
				end
			end
		}
		log.close unless @logdir.nil?
		retval = -1
		begin
			retval = File.read("/var/run/avupdate.ret").strip.to_i
		rescue
		end
		puts "avupdate returned: " + retval.to_s
		unless pgbar.nil?
			pgbar.text = "Aktualisierung der Virensignaturen abgeschlossen"
			pgbar.fraction = 1.0
		end
		if retval == 0
			# touch the key file
			system("touch /AntiVir/rescue_cd.key")
			return true, nil
		elsif retval == 1
			# no update available - do nothing
			return true, nil
		else
			# Update failed
			# FIXME! Show dialog informing on the problem
			return false, "Die Aktualisierung der Virensignaturen ist fehlgeschlagen. Wahrscheinlichste Ursache ist eine falsche Einstellung bei der Internetverbindung."
		end
	end
	
	# FIXME: Use wrapper script
	# FIXME: Use settings!
	# FIXME: Proper handling of boot sector scan
	
	def scan_partition(p, pc=0, pgbar=nil, label=nil)
		if @killed == true
			pgbar.text = "Virenscan abgebrochen" unless pgbar.nil?
			return false
		end
		p.filecount(pgbar)
		p.mount 
		filect = 0
		lastfname = nil
		infected = Array.new
		last_pulse = Time.now.to_f
		unless pgbar.nil?
			pgbar.text = "Partition " + pc.to_s + " von " + @partcount.to_s + " - Starte Scan"
			pgbar.fraction = 0.0 
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
		end
		# run ruby scancl_wrapper.rb instead 
		# IO.popen("/AntiVir/scancl -a -z -s --showall --log=/dev/null '" + p.mount_point[0] + "'") { |l|
		scancl = "LANGUAGE=en_GB:en LC_ALL=en_GB.UTF-8 ruby scancl_wrapper.rb /var/run/scancl.#{p.uuid}.ret /AntiVir/scancl " + @qparams.join(" ") + " "
		scancl = scancl + "'#{p.mount_point[0]}'" unless @params[-1] =~/^\// 
		puts "Running command: " + scancl
		now = Time.new.strftime("%Y%m%d-%H%M%S") 
		log = File.new(@logdir + "/" + now + "-" + p.device + "-botscan.txt", "w") unless @logdir.nil?
		# For debugging:
		fullog = nil
		if @logdir.nil? && @debug == true
			fulllog = File.new("/tmp/" + now + "-" + p.device + "-botscan-full.txt", "w")
		elsif AntibotMisc.boot_options["debug"] == true
			fulllog = File.new(@logdir + "/" + now + "-" + p.device + "-botscan-full.txt", "w")
		end
		puts "Logging to: " + @logdir + "/" + now + "-" + p.device + "-botscan.txt" unless @logdir.nil?
		prefix = Array.new
		prefix_finished = false
		suffix = Array.new
		suffix_started = false
		file_lines = Array.new
		alert = false
		last_count_change = Time.now.to_f
		last_count = 0
		IO.popen(scancl) { |l|
			while l.gets
				now = Time.now.to_f
				line = $_
				unless line.strip =~ /^-TickTock-$/ || fulllog.nil?
					fulllog.write line
					fulllog.flush
				end
				if prefix_finished == false
					if line.strip =~ /^\//
						prefix_finished = true
						unless @logdir.nil?
							prefix.each { |l| log.write(l) }
							0.upto(3) { |n| log.write("\n") }
						end
					else
						prefix.push(line) unless line.strip =~ /^-TickTock-$/
					end
				end
				
				if line.strip =~ /^Statistics/ || suffix_started == true
					if line.strip =~ /^Statistics/ && alert == true
						# Flush remaining alert
						file_lines.each { |l| log.write(l)  unless @logdir.nil? }
						alert = false
					end
					if line.strip =~ /^Statistics/
						0.upto(3) { |n| log.write("\n") unless @logdir.nil? }
					end
					unless line.strip =~ /^-TickTock-$/
						log.write(line) unless @logdir.nil?
						suffix.push(line)
					end
					suffix_started = true
				end
				
				if line.strip =~ /^\//
					lastfname = line.strip 
					filect += 1
					if file_lines.size > 0 && alert == true
						file_lines.each { |l| log.write(l) unless @logdir.nil? }
						alert = false
					end
					file_lines = Array.new
					file_lines.push(line)
				elsif (line.strip =~ /^-TickTock-$/ ).nil? 
					if line.strip =~ /^ALERT/
						alert = true 
						# Fileout
						fileout = nil
						fileout = ` file -b '#{lastfname}' `.strip if system("which file")
						infected.push AntibotInfectedFile.new(lastfname, p.mount_point[0], p.uuid, line, @qparams, fileout, @logdb, @tstamp)
						puts line.strip 
					end
					file_lines.push(line) unless suffix_started == true || prefix_finished == false 
				end
				
				unless pgbar.nil? || now - last_pulse < 0.1 
					last_pulse = now
					# last_count = filect.to_i
					unless p.filecount(pgbar).nil?
						if filect.to_i > 0
							if filect.to_i > last_count
								last_count = filect.to_i
								last_count_change = now
								pgbar.fraction = filect.to_f / p.filecount(pgbar).to_f
								pgtext = "Partition " + pc.to_s + " von " + @partcount.to_s + " - " + filect.to_s + " Dateien gescannt"
								pgtext = pgtext + " - " + infected.size.to_s + " Schädlinge" if infected.size > 1
								pgtext = pgtext + " - 1 Schädling" if infected.size == 1
								pgbar.text = pgtext
							elsif now - last_count_change > 3.0
								pgbar.pulse
								chopped_fname = lastfname.to_s.split("/")[-1] 
								pgbar.text = "Scanne Datei #{chopped_fname}"
							end
						else 
							pgbar.pulse
						end
					end
					while (Gtk.events_pending?)
						Gtk.main_iteration
					end
				end
			end
		}
		log.close unless @logdir.nil?
		unless pgbar.nil?
			pgbar.text = "Partition " + pc.to_s + " von " + @partcount.to_s + " - Scan abgeschlossen"
			pgbar.fraction = 1.0 
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
		end
		p.umount
		@infections[p] = infected
		@return_codes[p] = File.read("/var/run/scancl.#{p.uuid}.ret").strip.to_i 
		puts "Scanning partition: #{p.device} #{p.uuid} terminated with code #{@return_codes[p]} "
		fulllog.close unless fulllog.nil?
		return false if @killed == true
	end
	
	def run_scan(pgbar=nil, label=nil)
		# Reset variable for kill
		@killed = false
		pc = 1
		failures = Array.new
		( @drivescreen.int_drives + @drivescreen.ext_drives ).each { |d|
			d.partitions.each { |p|
				if @drivescreen.saved_states[p.uuid] == true
					scan_partition(p, pc, pgbar, label)
					pc += 1
				end
			}
		}
		if failures.size > 0
			# FIXME: Show error message indicating failed scan!
		end
	end
	
	def dump_scan_results
		
	end
	
	def dump_html_log
		return true if @logdb.nil?
		csslines = Array.new
		jslines = Array.new
		File.open("scanlog.css").each { |line| csslines.push(line.strip) }
		File.open("scanlog.js").each { |line| jslines.push(line.strip) } 
		# Read some rquired information
		dmiinfo = Array.new
		IO.popen("dmidecode --type system") { |l|
			while l.gets
				dmiinfo.push($_)
			end
		}
		# Build XML document 
		doc = REXML::Document.new(nil, { :respect_whitespace => %w{ script style } } )
		root = REXML::Element.new "html"
		body = REXML::Element.new "body"
		body.add_attribute("onload", "jsinit();") 
		head = REXML::Element.new "head"
		m1 = REXML::Element.new "meta"
		m1.add_attribute("http-equiv", "Content-Type")
		m1.add_attribute("content", "text/html; charset=UTF-8")
		css = REXML::Element.new "style"
		css.add_attribute("type", "text/css")
		# csslines.each { |l| css.add_text l + "\n" }
		css.add(REXML::Text.new("\n" + csslines.join(" \n ") + "\n", true, nil, true ))
		js = REXML::Element.new "script"
		js.add_attribute("type", "text/javascript")
		# jslines.each { |l| js.add_text l + "\n" }
		js.add(REXML::Text.new("\n" + jslines.join(" \n ") + "\n", true, nil, true ))
		head.add m1
		head.add css
		head.add js 
		t = REXML::Element.new "title"
		t.add_text "Protokoll des Virenscans"
		head.add t
		root.add head
		root.add body
		doc.add root
		# Header section with general information
		allhead = REXML::Element.new "h1"
		allhead.add_text "Protokoll des Virenscans"
		genhead = REXML::Element.new "h2"
		genhead.add_text "Allgemeine Informationen"
		body.add allhead
		body.add genhead
		pgen = REXML::Element.new "p"
		pgen.add_attribute("class", "pgen")
		pgen.add_text "Datum: " + Time.now.to_s 
		pgen.add REXML::Element.new "br"
		pgen.add_text "Verwendete Parameter: " + @params.join(" ")
		body.add pgen
		# div boxes "Computer" and "Avira"
		divgen = REXML::Element.new "div"
		agen = REXML::Element.new "a" 
		agen.add_text "Informationen zum Computer ausklappen"
		agen.add_attribute("href", "#divgen")
		agen.add_attribute("id", "showcomp")
		age2 = REXML::Element.new "a" 
		age2.add_text "Informationen zum Computer einklappen"
		age2.add_attribute("href", "#divgen")
		age2.add_attribute("id", "hidecomp")
		divgen.add agen
		divgen.add REXML::Element.new "br"
		divgen.add age2
		dmipre = REXML::Element.new "div"
		dmiinfo.each { |l|
			dmipre.add_text l
			dmipre.add REXML::Element.new "br"
		}
		dmipre.add_attribute("id", "dmipre")
		divgen.add dmipre
		divavr = REXML::Element.new "div"
		aavr = REXML::Element.new "a" 
		aavr.add_text "Informationen zu Avira ausklappen"
		aavr.add_attribute("href", "#avrgen")
		aavr.add_attribute("id", "showavira")
		aav2 = REXML::Element.new "a" 
		aav2.add_text "Informationen zu Avira einklappen"
		aav2.add_attribute("href", "#avrgen")
		aav2.add_attribute("id", "hideavira")
		divgen.add aavr
		divgen.add REXML::Element.new "br"
		divgen.add aav2
		avrpre = REXML::Element.new "div"
		@avira_version.each { |l|
			avrpre.add_text l
			avrpre.add REXML::Element.new "br"
		}
		avrpre.add_attribute("id", "avrpre")
		divgen.add avrpre
		body.add divgen
		# div box for scanned drives
		divdrv = REXML::Element.new "div"
		divdrv.add_attribute("id", "divdrives")
		divext = REXML::Element.new "div"
		divext.add_attribute("id", "extdrives")
		divext.add_attribute("class", "intextdrives")
		divint = REXML::Element.new "div"
		divint.add_attribute("id", "intdrives")
		divint.add_attribute("class", "intextdrives")
		hint = REXML::Element.new "h3"
		hint.add_text "Interne Laufwerke"
		hext = REXML::Element.new "h3"
		hext.add_text "Externe Laufwerke"
		divint.add hint
		divext.add hext
		divdrv.add divint
		divdrv.add divext
		body.add divdrv
		# General Info for each drive
		int_divs = Hash.new
		ext_divs = Hash.new
		@drivescreen.int_drives.each { |d|
			int_divs[d.device] =  REXML::Element.new "div"
			int_divs[d.device].add_attribute("id", "drive-#{d.device}")
			int_divs[d.device].add_attribute("class", "drivebox")
			divint.add int_divs[d.device]
		}
		@drivescreen.ext_drives.each { |d|
			ext_divs[d.device] =  REXML::Element.new "div"
			ext_divs[d.device].add_attribute("id", "drive-#{d.device}")
			ext_divs[d.device].add_attribute("class", "drivebox")
			divext.add ext_divs[d.device]
		}
		( @drivescreen.int_drives + @drivescreen.ext_drives ).each { |d|
			h = REXML::Element.new "h4"
			h.add_text d.vendor + " " + d.model
			# Link to fold SMART info
			a = REXML::Element.new "a"
			a.add_text "SMART-Informationen ausklappen"
			a.add_attribute("href", "#" + d.device + "-unfold")
			a.add_attribute("id", "showsmart-" + d.device)
			a.add_attribute("class", "showsmart")
			a.add_attribute("onclick", 'showsmart("' + d.device + '");')
			b = REXML::Element.new "a"
			b.add_text "SMART-Informationen einklappen"
			b.add_attribute("href", "#" + d.device + "-unfold")
			b.add_attribute("id", "hidesmart-" + d.device)
			b.add_attribute("class", "hidesmart")
			b.add_attribute("onclick", 'hidesmart("' + d.device + '");')
			# Smart information
			smart_adiv = REXML::Element.new "div"
			smart_adiv.add_attribute("class", "smartouter")
			smart_adiv.add_attribute("id", "smart-" + d.device)
			smart_idiv = REXML::Element.new "div"
			smart_idiv.add_attribute("class", "smartinfo")
			smart_ediv = REXML::Element.new "div"
			smart_ediv.add_attribute("class", "smarterror")
			smart_ipre = REXML::Element.new "div"
			smart_epre = REXML::Element.new "div"
			smart_summ = REXML::Element.new "div"
			smart_summ.add_attribute("class", "smartsummary")
			s, i, e = d.error_log
			i.each { |l|
				smart_ipre.add_text l
				smart_ipre.add REXML::Element.new "br"
			}
			e.each { |l|
				smart_epre.add_text l
				smart_epre.add REXML::Element.new "br"
			}
			smart_idiv.add smart_ipre
			smart_ediv.add smart_epre
			smart_adiv.add a
			smart_adiv.add REXML::Element.new "br"
			smart_adiv.add b
			smart_adiv.add smart_idiv
			if s == true
				if e.size < 1
					smart_summ.add_text "Es sind keine Fehler im SMART-Log gespeichert"
				else
					smart_summ.add_text "Es sind Fehler im SMART-Log gespeichert - tauschen Sie die Festplatte bald aus!"
				end
			else
				smart_summ.add_text "Die Festplatte unterstützt SMART nicht!"
			end
			smart_adiv.add smart_summ
			smart_adiv.add smart_ediv if e.size > 0
			# Now Partition by partition
			pboxes = Array.new
			d.partitions.each { |p|
				pouterbox = REXML::Element.new "div"
				pouterbox.add_attribute("class", "pouterbox")
				pinnerbox = REXML::Element.new "div"
				pinnerbox.add_attribute("class", "pinnerbox")
				pshort = REXML::Element.new "div"
				pshort.add_attribute("class", "psummary")
				phead = REXML::Element.new "h5"
				windows, winvers = p.is_windows
				pht = p.device.to_s + ", " + p.fs.to_s + " " + p.label.to_s + " " + p.human_size
				pht = pht + " - " + winvers if windows == true
				phead.add_text pht
				pouterbox.add phead
				pouterbox.add pshort
				if @infections[p].nil?
					pshort.add_text "Diese Partition wurde nicht gescannt"
				elsif @infections[p].size > 0
					pshort.add_text "Diese Partition wurde gescannt - es wurde Schadsoftware gefunden!"
					# retrieve table with infected files
					pinnerbox.add infected_table(p)
					pouterbox.add pinnerbox
				else
					pshort.add_text "Diese Partition wurde gescannt - es wurde keine Schadsoftware gefunden"
				end
				pboxes.push pouterbox if p.fs =~ /ext/ || p.fs =~ /fat/ || p.fs =~ /ntfs/ || p.fs =~ /btrfs/ 
			}
			if ext_divs.has_key? d.device
				ext_divs[d.device].add h
				ext_divs[d.device].add smart_adiv
				pboxes.each { |pb| ext_divs[d.device].add pb }
			else
				int_divs[d.device].add h
				int_divs[d.device].add smart_adiv
				pboxes.each { |pb| int_divs[d.device].add pb }
			end
		}
		# @logdir @tstamp
		unless @logdir.nil?
			outfile = File.new(@logdir + "/scanprotokoll-" + @tstamp.to_s + ".html", File::CREAT|File::TRUNC|File::RDWR) 
		else
			system("mkdir -p /tmp/Protokolle")
			outfile = File.new("/tmp/Protokolle/scanprotokoll-" + @tstamp.to_s + ".html", File::CREAT|File::TRUNC|File::RDWR)
		end
		doc.write(outfile) # , 4)
		outfile.close
	end
	
	def infected_table(p)
		inftab = REXML::Element.new "table"
		inffirst = REXML::Element.new "tr"
		[ "Fundort", "Schadsoftware", "durchgeführte Aktion" ].each { |i|
			td = REXML::Element.new "td"
			td.add_text i
			inffirst.add td
		}
		thead = REXML::Element.new "thead"
		thead.add inffirst
		inftab.add thead
		tbody = REXML::Element.new "tbody"
		lct = 0
		infections[p].each { |i|
			tr = REXML::Element.new "tr"
			fn = REXML::Element.new "td"
			ic = REXML::Element.new "td"
			ac = REXML::Element.new "td"
			il = REXML::Element.new "a"
			il.add_attribute("href", "http://www.avira.com/de/support-virus-lab?sq=" + i.short_infection)
			il.add_attribute("target", "_blank")
			fn.add_text i.relpath
			ic.add il
			il.add_text i.short_infection
			if i.action_taken == "desinfect" 
				ac.add_text "desinfiziert"
			elsif i.action_taken == "delete" 
				ac.add_text "gelöscht"
			elsif i.action_taken == "truncate" 
				ac.add_text "gelöscht (auf Größe Null geschrumpft)"
			else
				ac.add_text "keine"
			end
			tr.add fn
			tr.add ic
			tr.add ac
			tr.add_attribute("class", "oddline") if lct % 2 == 1
			tr.add_attribute("class", "evenline") if lct % 2 == 0
			tbody.add tr			
			lct += 1
		}
		inftab.add tbody
		return inftab 
	end
	
end
