#!/usr/bin/ruby
# encoding: utf-8

require 'gtk2'
require 'vte'
require "rexml/document"
require 'optparse'

require "MfsDiskDrive.rb"
require "MfsSinglePartition.rb"
require "AntibotDriveScreen.rb"
require "AntibotOptionScreen.rb"
require "AntibotScanFrame.rb"

# If there is text in the GtkEntry, set the page as complete.
# Otherwise, stop the user from progressing the next page.
def entry_changed (entry, assistant)
  text = entry.text
  num = assistant.current_page
  page = assistant.get_nth_page(num)
  assistant.set_page_complete(page, (text.size > 0))
end

# If the check button is toggled, set the page as complete.
# Otherwise, stop the user from progressing the next page.
def button_toggled(toggle, assistant)
  num = assistant.current_page
  page = assistant.get_nth_page(num)
  assistant.set_page_complete(page, toggle.active?)
end

# Fill up the progress bar, 10% every second when the button
# is clicked. Then, set the page as complete when the progress
# bar is filled.
def button_clicked (button, assistant, progress)
  percent = 0.0
  button.sensitive = false
  num = assistant.current_page
  page = assistant.get_nth_page(num)
  while(percent <= 100.0)
    progress.text = "%.0f%% abgeschlossen" % percent
    progress.fraction = percent / 100.0
    while (Gtk.events_pending?)
      Gtk.main_iteration
    end
    sleep 0.2
    percent += 5.0
  end
  assistant.set_page_complete(page, true)
end

# This function is where you would apply the changes and
# destroy the assistant.
def assistant_close (assistant)
  puts "You would apply your changes now!\n"
  assistant.destroy
end

class PageInfo
  attr_accessor :widget, :index, :title, :type, :complete
  def initialize(w, i, ti, ty, c)
    @widget, @index, @title, @type, @complete = w, i, ti, ty, c
  end
end

page = [
  PageInfo.new(nil, -1, "Start", Gtk::Assistant::PAGE_INTRO,    true),
  PageInfo.new(nil, -1, "Auswahl der Laufwerke", Gtk::Assistant::PAGE_CONTENT,  true),
  # PageInfo.new(nil, -1, "Scan-Optionen", Gtk::Assistant::PAGE_CONTENT,  true),
  # Bereits inklusive
  # PageInfo.new(nil, -1, "Introduction",            Gtk::Assistant::PAGE_INTRO,    true),
  # PageInfo.new(nil, -1, "",                        Gtk::Assistant::PAGE_CONTENT,  false),
  # PageInfo.new(nil, -1, "Click the Check Button",  Gtk::Assistant::PAGE_CONTENT,  false),
  # Scan Page
  PageInfo.new(nil, -1, "Virusscan durchführen",        Gtk::Assistant::PAGE_PROGRESS, false),
  PageInfo.new(nil, -1, "Computer reinigen", Gtk::Assistant::PAGE_CONTENT,  true),
  
  
  
  
  PageInfo.new(nil, -1, "Reinigung abgeschlossen",            Gtk::Assistant::PAGE_CONFIRM,  true)
]

# Create a new assistant widget with no pages. */
assistant = Gtk::Assistant.new
assistant.set_size_request(650, 450)   # 450, 300
assistant.signal_connect('destroy') { Gtk.main_quit }

# Populate the pages:
# Page 0 contains an overview of detected settings...

# page[0].widget = Gtk::Label.new("Netzwerk, Akku, Arbeitsspeicher")
stab = Gtk::Table.new(4, 3, false)
stab.attach_defaults(Gtk::Button.new("Einstellungen"), 3, 4, 0, 1)
stab.attach_defaults(Gtk::Button.new("Einstellungen"), 3, 4, 2, 3)

iconone = Gtk::Image.new(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
stab.attach_defaults(iconone, 0, 1, 0, 1)
nt1 = Gtk::Label.new("<b>Netzwerk</b>")
nt1.use_markup = true
stab.attach_defaults(nt1, 1, 2, 0, 1)
ntl = Gtk::Label.new("Die automatisch erkannte Netzwerkverbindung kann zur Aktualisierung der Virensignaturen benutzt werden")
ntl.wrap = true
stab.attach_defaults(ntl, 2,3,0, 1)
pw1 = Gtk::Label.new("<b>Energie</b>")
pw1.use_markup = true
stab.attach_defaults(pw1, 1,2,1, 2)
pwr = Gtk::Label.new("Ihr Notebook ist an ein Ladegerät angeschlossen oder Sie benutzen einen Desktop-PC")
pwr.wrap = true
icontwo = Gtk::Image.new(Gtk::Stock::APPLY, Gtk::IconSize::BUTTON)
stab.attach_defaults(icontwo, 0, 1, 1, 2)
stab.attach_defaults(pwr, 2, 3, 1, 2)
ra1 = Gtk::Label.new("Ihr Computer verfügt über wenig Arbeitsspeicher, ein USB-Speicherstift ist zwingend erforderlich")
ra1.wrap = true
ram = Gtk::Label.new("<b>Speicher</b>")
ram.use_markup = true
stab.attach_defaults(ra1, 2, 3, 2, 3)
stab.attach_defaults(ram, 1, 2, 2, 3)
iconthree = Gtk::Image.new(Gtk::Stock::CANCEL, Gtk::IconSize::BUTTON)
stab.attach_defaults(iconthree, 0, 1, 2, 3)
page[0].widget = stab

# In page 1 you tell which drives to scan:

dscrl = Gtk::ScrolledWindow.new
dscrn = AntibotDriveScreen.new
dscrl.add_with_viewport dscrn.wdgt
dscrl.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC)
page[1].widget = dscrl # dbox # Gtk::Label.new("Auswahl der Laufwerke, weiter zur Aktualisierung")

# In page 2 we will set some options for the scan
# FIXME: Remove!
# page[2].widget = create_conftab[0] # Gtk::Label.new("Scan-Optionen, weiter zum Scan")

# In Page 3 we do the virus scan with a nice progress bar

oscrn = AntibotOptionScreen.new
scnfr = AntibotScanFrame.new
svbox = Gtk::VBox.new(false, 5)
svbox.pack_start(oscrn.wdgt, false, false, 5)
button = Gtk::Button.new("Los geht's")
progress = Gtk::ProgressBar.new
progress.width_request = 500
hbox = Gtk::HBox.new(false, 5)
hbox.pack_start(progress, true, false, 5)
hbox.pack_start(button,   false, false, 5)
# svbox.pack_start(hbox, true, false, 5)
svbox.pack_start(scnfr.wdgt, false, false, 5)
page[2].widget = svbox



# In page 4 we provide a summary of the scan and offer an option to clean

sbox = Gtk::VBox.new(false, 5)
sstat = Gtk::HBox.new(false, 5)
slabel = Gtk::Label.new("<b>Es wurde Schadsoftware gefunden!</b>")
slabel.use_markup = true
sbox.pack_start(slabel)
sdet = Gtk::Label.new("Es wurde Schadsoftware auf einem oder mehreren Laufwerken gefunden. Voraussichtlich kann die Schadsoftware ohne Beeinträchtigungen der Nutzbarkeit Ihres Computers entfernt werden. Für Analyse- und Wiederherstellungszwecke wird eine Sicherung der geänderten Daten auf USB-Stick angelegt.\n\nKlicken Sie auf \"Weiter\" um die Reinigung vorzunehmen. Die gesicherten Dateien werden auf dem zu Beginn des Vorgangs angegebenen USB-Laufwerkes gespeichert.")
sdet.wrap = true
sdet.use_markup = true
sstat.pack_start(Gtk::Image.new(Gtk::Stock::DIALOG_WARNING, Gtk::IconSize::BUTTON))
sstat.pack_start(sdet)
sbox.pack_start(sstat)

page[3].widget = sbox # Gtk::Label.new("Optionen fürs Reinigen auswählen, Datensicherung")

# In page 5 we provide a short summary...

p5b =  Gtk::VBox.new(false, 5)
l1 = Gtk::Label.new("Die Reinigung wude beendet. Eine Sicherung der gereinigten Dateien finden Sie auf dem angegebenen USB-Speicher im Ordner \"Antibot_Sicherung\".\n\nBitte helfen Sie dem Antibot-Team, indem Sie eine Statistik der Virensuche übertragen. Dies ist freiwillig und es werden nur die Namen der gefundenen Schädlinge, aber keine Dateiinhalte übertragen. Wenn Sie bereit sind, Rückfragen zu beantworten, geben Sie Ihre Email-Adresse an.")
l1.wrap = true
p5b.pack_start(l1)
p5b.pack_start(Gtk::CheckButton.new("Protokoll übertragen"))
p5b.pack_start(Gtk::Entry.new)

page[4].widget = p5b
# 
      
# Create the necessary widgets for the second page.
label = Gtk::Label.new("Your Name: ")
entry = Gtk::Entry.new


# Add five pages to the Gtk::Assistant dialog.
5.times do |i|
  page[i].index = assistant.append_page(page[i].widget) 
  assistant.set_page_title(page[i].widget, page[i].title)
  assistant.set_page_type(page[i].widget, page[i].type)

  # Set the introduction and conclusion pages as complete so
  # they can be incremented or closed.
  assistant.set_page_complete(page[i].widget, page[i].complete)
end

# Update whether pages 2 through 4 are complete based upon
# whether there is text in the GtkEntry, the check button is
# active, or the progress bar is completely filled.

entry.signal_connect(         'changed') { |w| entry_changed(w, assistant) }
# page[2].widget.signal_connect('toggled') { |w| button_toggled(w, assistant) }
button.signal_connect(        'clicked') { |w| button_clicked(w, assistant, progress) }
assistant.signal_connect(     'cancel')  { assistant.destroy }
assistant.signal_connect(     'close')   { |w| assistant_close(w) }

assistant.show_all
Gtk.main